/*
 * Copyright 2018-2020 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 * 1. Create the KF memory region
 * 2. Create the display memory region
 * 3. Place the VM stacks, LLCOM_SPI buffers and images heap in PSRAM
 * 4. Place the Java heap, immortals, ... in PSRAM
 */

/* MicroEJ buffers sizes */
__KERNEL_WORKING_BUFFER_Size__      = 0x100000;

/* (320 * 240 * 2) x 1 : 1 for BACK_BUFFER */
__DISPLAY_MEM_Size__                = 0x25800;

/* This section is only included if CONFIG_SPIRAM_ALLOW_BSS_SEG_EXTERNAL_MEMORY
   is set, to link some sections to BSS in PSRAM */

SECTIONS
{
   ._kf_stack :
   {
    . = ALIGN(4);
    PROVIDE ( __KERNEL_WORKING_BUFFER_Start__ = . );
    . = . + __KERNEL_WORKING_BUFFER_Size__;
    . = ALIGN(4);
    __KERNEL_WORKING_BUFFER_End__ = .;         /* define a global symbol at KF memory end */
   } > extern_ram_seg

   ._display_stack :
   {
    . = ALIGN(4);
    PROVIDE ( __DISPLAY_MEM_Start__ = . );
    . = . + __DISPLAY_MEM_Size__;
    . = ALIGN(4);
    __DISPLAY_MEM_End__ = .;         /* define a global symbol at DISPLAY memory end */
   } > extern_ram_seg

   /* external memory bss, from any global variable with EXT_RAM_ATTR attribute*/
  .ext_ram.bss (NOLOAD) :
  {
    _ext_ram_bss_start = ABSOLUTE(.);
    *(.ext_ram.bss*)
    *libnet80211.a:(.dynsbss .sbss .sbss.* .gnu.linkonce.sb.* .scommon .sbss2.* .gnu.linkonce.sb2.* .dynbss .bss .bss.* .share.mem .gnu.linkonce.b.* COMMON)
    *libpp.a:(.dynsbss .sbss .sbss.* .gnu.linkonce.sb.* .scommon .sbss2.* .gnu.linkonce.sb2.* .dynbss .bss .bss.* .share.mem .gnu.linkonce.b.* COMMON)
    *liblwip.a:(.dynsbss .sbss .sbss.* .gnu.linkonce.sb.* .scommon .sbss2.* .gnu.linkonce.sb2.* .dynbss .bss .bss.* .share.mem .gnu.linkonce.b.* COMMON)
    *libbt.a:(EXCLUDE_FILE (libbtdm_app.a) .dynsbss .sbss .sbss.* .gnu.linkonce.sb.* .scommon .sbss2.* .gnu.linkonce.sb2.* .dynbss .bss .bss.* .share.mem .gnu.linkonce.b.* COMMON)
    . = ALIGN(4);

    *(ICETEA_HEAP)
    *(_java_heap)
    *(_java_immortals)
    *(.bss.vm.stacks.*)
    *(.bss.microui.display.imagesHeap)
    *(.psram*)

    _ext_ram_bss_end = ABSOLUTE(.);
  } > extern_ram_seg
}
