/*
 * C
 *
 * Copyright 2018-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 *
 */

#include <stdlib.h>

#include "esp_heap_caps.h"

#include "microej_allocator_configuration.h"
#include "microej_allocator.h"

#include "esp_log.h"
void* microej_calloc4tls(size_t nmemb, size_t size) {
	//ESP_LOGI(__func__, "allocate %d, %d\n", nmemb, size);
	return microej_calloc(nmemb, size);
}

void microej_free4tls(void *ptr) {
	//ESP_LOGI(__func__, "free %p\n", ptr);
	microej_free(ptr);;
}

void* microej_malloc(size_t size)
{
#ifdef CONFIG_MICROEJ_ALLOCATION_FROM_SPIRAM_FIRST
    return heap_caps_malloc_prefer(size, 2, MALLOC_CAP_DEFAULT|MALLOC_CAP_SPIRAM, MALLOC_CAP_DEFAULT|MALLOC_CAP_INTERNAL);
#else
    return malloc(size);
#endif
}

void* microej_calloc(size_t nmemb, size_t size)
{
#ifdef CONFIG_MICROEJ_ALLOCATION_FROM_SPIRAM_FIRST
    return heap_caps_calloc_prefer(nmemb, size, 2, MALLOC_CAP_DEFAULT|MALLOC_CAP_SPIRAM, MALLOC_CAP_DEFAULT|MALLOC_CAP_INTERNAL);
#else
    return calloc(nmemb, size);
#endif
}

void microej_free(void *ptr)
{
#ifdef CONFIG_MICROEJ_ALLOCATION_FROM_SPIRAM_FIRST
	return heap_caps_free(ptr);
#else
	return free(ptr);
#endif
}

